/*
  (C) COPYRIGHT International Business Machines Corp., 2011. 
   All Rights Reserved * Licensed Materials - Property of IBM 
 */

// dojo.provide allows pages to use all of the types declared in this resource.
dojo.provide("widgets.panel.CollectionStatus");

// dojo.require the necessary dijit hierarchy
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.parser");
dojo.require("dijit.ProgressBar");
dojo.require("dojox.form.BusyButton");
dojo.require("widgets.customTasks.UploadInterface");
dojo.require("dojo.i18n");

//Loading of nls variables
dojo.requireLocalization("widgets.panel", "panelstrings");

dojo.declare("widgets.panel.CollectionStatus", [dijit._Widget, dijit._Templated ], {
	
	// Path to the template
	templateString: dojo.cache("widgets", "panel/templates/CollectionStatus.html"),

	// Set this to true if your widget contains other widgets
	widgetsInTemplate : true,
	
	//This is the message displayed in the status area
	statusMessage:'',
	
	//This is the default increment after each action on the status
	statusIncrement:1,
	
	//Depends on our execution protocol: file or http
	execEnvIsFile:true,
	
	//Previous archive location
	_archiveLocation: "",
	
	//Variable to keep track of the current UploadInterface
	uploadInterface: null,

	// Override this method to perform custom behavior during dijit construction.
	// Common operations for constructor:
	// 1) Initialize non-primitive types (i.e. objects and arrays)
	// 2) Add additional properties needed by succeeding lifecycle methods
	constructor : function(/* Object */args) {
		// summary: Constructor for the search widget. It allows specifying of attributes.
		dojo.mixin(this,args);
	},

	// When this method is called, all variables inherited from superclasses are 'mixed in'.
	// Common operations for postMixInProperties
	// 1) Modify or assign values for widget property variables defined in the template HTML file
	postMixInProperties : function() {		
		//do the nls mixin here
		var _nlsResources=dojo.i18n.getLocalization("widgets.panel","panelstrings");
		
		dojo.mixin(this,_nlsResources);


		this.inherited(arguments);
	},
	
	/**
	 * Method to initialize the widget values after it is created.
	 */ 
	postCreate: function() {
		this.inherited("postCreate Collection Status ...", arguments);
		
		 //This widget subscribes to update status events
	     dojo.subscribe("UpdateStatusArea", dojo.hitch(this,function(item){
	    	 console.debug("Collection UpdateStatus Area event subscription... "+item);
	    	 this.updateStatusNow(item, false);
	    	 this._enableBusy(); //Enable the busy signal
	     }));
	     
	     //This widget subscribes to collection completion events.
	     dojo.subscribe("CollectionCompleted", dojo.hitch(this,function(item){
	    	 this._disableBusy(); //Disable the busy signal
	    	 console.debug("Collection Completed event subscription... "+item);
	    	 //dijit.byId("statusCancelButtonID").set("disabled",true);
	    	 this.updateStatusNow(item, true);
	    	 this.publishUploadInterface(item);
	     }));
	     
	     //This widget subscribes to collection completion events.
	     dojo.subscribe("UploadStatusCompleted", dojo.hitch(this,function(item){
			this._disableBusy(); //Disable the busy signal
			this.updateStatusNow(item, true);
	     }));

	     dojo.subscribe("collectionStarted",dojo.hitch(this,function(item){
	       console.debug("collection started ...");	       
	       this._resetStatus(); //Enable the Spinner Icon
	       this._enableBusy(); //Enable the Spinner Icon	       
	     }));
	     
	     dojo.subscribe("inputDialogResponseProvided", dojo.hitch(this,function(response){
	    	 console.debug("Response submitted ...");
	    	 this._enableBusy(); //Enable the Spinner Icon
	     }));
	     
	      dojo.subscribe("InputRequested", dojo.hitch(this,function(response){
	    	 console.debug("Input Requested ...");
	    	 this._disableBusy(); //Enable the Spinner Icon
	     }));
	      
		 //This widget subscribes to Set Archive Location events
	     dojo.subscribe("SetArchiveLocation", dojo.hitch(this,function(item){
	    	 this.setArchiveLocation(dojo.fromJson(item).location);
	     }));
	     
	     
	     dojo.subscribe("ShowUploadInterface", dojo.hitch(this,function(item){
	    	 this.statusMessage = this.transferMessage;
	    	this._updateStatus();
	     }));
	 
	     var busyButton= new dojox.form.BusyButton({
	           id:'busyIndicatorID',
	           busyLabel:this.collectingBusyIconLabel,
	           alt:this.collectingBusinIconLabel});
	       
		this.busyIconButton.appendChild(busyButton.domNode);
	},
	//Update all the status related variable.
	//Calculate the various end statuses based on the collectinStatus object
	updateStatusNow: function(/*JSONString*/ jsonStatus, completed){	
		//
		//summary: This updates the status 
		//
		var statusObj = dojo.fromJson(jsonStatus);
		
		if(!completed){
			if(statusObj.message) {
				this.statusMessage=statusObj.message;
			}
		  if (statusObj.progress) {
			  this.statusIncrement=statusObj.progress;
		  }
		  
		} else {
			var canceled="";
			var failure="";
			var recovery="";
			var status="";
			var noUpload="";
			
			if (statusObj.collectionStatus.canceled) {
				canceled=statusObj.collectionStatus.canceled;
			}
			if (statusObj.collectionStatus.noUpload) {
				noUpload=statusObj.collectionStatus.noUpload;
			}
			if (statusObj.collectionStatus.collectionArchive){
				this.setArchiveLocation(statusObj.collectionStatus.collectionArchive);
			}
			if (statusObj.collectionStatus.collectionFailure) {
				failure=statusObj.collectionStatus.collectionFailure;
			}
			if (statusObj.collectionStatus.recoveryFailure){
				recovery=statusObj.collectionStatus.recoveryFailure;
			}
			if (statusObj.collectionStatus.completionStatus) {
				status=statusObj.collectionStatus.completionStatus;
			}

			
			this.statusIncrement=100;
			
			var displayName=this._archiveLocation.replace(/\\\\/gi,'\\');
			if(canceled){ // User has canceled the Collection
				if (statusObj.type=="UPLOAD") {
					if (this.execEnvIsFile) {
						var parts = displayName.match(/(.*)[\/\\]([^\/\\]+\.\w+)$/);
						this.statusMessage=this.cancelUploadByUser + displayName+"&nbsp;&nbsp;(<a href='file:///"+ parts[1] +"' target='_blank'>" +this.browseDirectory + " </a>)";
					}
					else {
						this.statusMessage=this.cancelUploadByUser + displayName;
					}
				}
				else {
					this.statusMessage=this.collectionCanceledByUser;	
				}  
			}
			if (noUpload) {
				if (this.execEnvIsFile) {
					var parts = displayName.match(/(.*)[\/\\]([^\/\\]+\.\w+)$/);
					this.statusMessage=this.doNotUploadByUser + displayName+"&nbsp;&nbsp;(<a href='file:///"+ parts[1] +"' target='_blank'>" +this.browseDirectory + " </a>)";
				}
				else {
					this.statusMessage=this.doNotUploadByUser + displayName;
				}
			}
			else if(status=="NORMAL" && failure==false && canceled==false){
				var linkToFile=this._archiveLocation.replace(/\\\\/gi,'/');
				
				if (this.execEnvIsFile) {
					var parts = linkToFile.match(/(.*)[\/\\]([^\/\\]+\.\w+)$/);

					if (statusObj.type=="UPLOAD") {
						//Upload is complete, show the restart button now
						this.enableRestartButton();
						this.statusMessage=this.uploadCompletionMessage + displayName+"&nbsp;&nbsp;(<a href='file:///"+ parts[1] +"' target='_blank'>" +this.browseDirectory + " </a>)";
					}
					else {
						this.statusMessage=this.collectionCompletionMessage;
					}
				} else {
					if (statusObj.type=="UPLOAD") {
						this.enableRestartButton();
						//Upload is complete, show the restart button now
						this.statusMessage=this.uploadCompletionMessage + displayName;
					}
					else {
						this.statusMessage=this.collectionCompletionMessage;
					}
				}
				this._addClass();
				//clear the uploadinterface
				this._publishCompleteSuccess();
				
			}else if(status=="ERROR"){
				console.debug("There was an error : " + statusObj.collectionStatus.collectionErrorMsg);
				if (!statusObj.collectionStatus.collectionErrorMsg) {
					var uploadMsg = statusObj.collectionStatus.message;
					uploadMsg = uploadMsg.replace(/\\\\/g,"\\");
					this.statusMessage = uploadMsg;
				}
				else {
					this.statusMessage=statusObj.collectionStatus.collectionErrorMsg;
				}
				this._addClass();
				
				if(!canceled){//Error occured during collection
					this._publishCompletedFailed();
					this.enableRestartButton();
					this.enableRestartUploadButton();
				}
			}else {
				this.enableRestartUploadButton();
			}			
		}
		
		//Update the elements with the new status
		this._updateStatus(completed);		
	},
	// Update the look and feel of the progress bar, depending on the completion status of the execution
	//
	_addClass: function(){
		// summary:
		//		This method updates the look and feel of the progress bar, when completion has been reached.
		// 
		//      If there was an error, the look and feel will look a certain way (red?)
		//      If the completion was successful and there was no error and no warning, the look and feel will look (green?)
		//      If there was warnings, the look and feel will look a certain way (yellow?)
		//		
		console.debug("Update this class to reflect the visual of the Progress bar"); 
		//TODO: update this class to reflect the visual of the Progress bar
	},
	_resetStatus: function() {
		this.statusMessage = "";
		this.statusIncrement=1;
		this._updateStatus();
	},
	//Effectively updates the status from the new values set
	//
	_updateStatus: function(completed){
		// summary:
		//		This method updates the status message and the progress bar (TODO: progress bar should be updated in a different way)
		// 
		//      Messages sent from the back end use this method to display on the User Interface. No changes are made to the messages
		//      The messages will be displayed exactly as received.
		//		
		  dojo.byId("statusMessageID").innerHTML = this.statusMessage;
		  dijit.byId("collectionProgress").progress=this.statusIncrement;	
		  dijit.byId("collectionProgress").update();
		  		  
		  console.debug("update status ..."+this.statusMessage);		  
	},
	// This method is called, when the Cancel Collection  button on the data collection is clicked.
	//
	cancelDataCollection: function(){
		// summary:
		//		This method is called, when the Cancel Collection button is clicked during data collection
		// 
		//		
		console.debug("cancel data collection ...");
	},
	//
	//
	_publishCompleteSuccess: function(){
		//
		// summary: publishes event that a build completed successfully
		//
	    dojo.publish("CollectionCompletedWithSuccess", [{'status':'success'}]);
	},
	//
	//
	//
	_publishCompleteCanceled: function(){
		//
		// summary: publishes event that a build was successfully canceled
		//
	    dojo.publish("CollectionCompletedCanceled", [{'status':'canceled'}]);
	},
	//
	//
	//
	_publishCompletedFailed: function(){
		//
		// summary: publishes event that a build failed
		//
	    dojo.publish("CollectionCompletedFailed", [{'status':'failed'}]);
	},

	//
	// Enables the busy indicator
	_enableBusy: function(){
		//
		// summary: This function enables the busy indicator
		//
		console.debug("enable_Busy ");
		dojo.style(dijit.byId("busyIndicatorID").domNode,"display","block");
		dijit.byId("busyIndicatorID").makeBusy();
	},
	//
	//Disables the busy Indicator
	_disableBusy: function(){
		//
		// summary: This function disables the busy indicator
		//
		console.debug("disable_Busy ");
		dojo.style(dijit.byId("busyIndicatorID").domNode,"display","none");
		//dijit.byId("busyIndicatorID").cancel();
	},
	
	//Builds the Upload Interface
	publishUploadInterface: function(collectionData) {
		
		var statusObj = dojo.fromJson(collectionData);
		this.setArchiveLocation(statusObj.collectionStatus.collectionArchive);
		
		var uploadPMRNumber = "";
		if (executionEnv.pmrNumber != "00000,000,000" && executionEnv.pmrNumber !="") {
			uploadPMRNumber = executionEnv.pmrNumber;
		}
		
		if(this.uploadInterface==null) {
			this.uploadInterface = new widgets.customTasks.UploadInterface({
				displayIBMTransfer:"true",
				displayOtherTransfer:"true",
				remoteFileName:"",
				pmrNumber:uploadPMRNumber,
				email:executionEnv.email
			});
			
			dojo.subscribe("CancelCurrentAction", function(){
				dojo.publish("CancelUpload", []);
				dojo.unsubscribe("CancelCurrentAction");
			});
			

			//Remove the Upload Interface from the GUI
			dojo.subscribe("RemoveUploadInterface", dojo.hitch(this, function() {
				this.uploadInterface.destroyRecursive();
			}));
			
			//Remove the Upload Interface from the GUI
			dojo.subscribe("HideUploadInterface", dojo.hitch(this, function() {
				this.uploadInterface.hide();
			}));
			
			//Remove the Upload Interface from the GUI
			dojo.subscribe("ShowUploadInterface", dojo.hitch(this, function() {
				this.uploadInterface.show();
			}));
			
			dojo.connect(this.uploadInterface.sendUploadButtonNode, "onClick", dojo.hitch(this, function(){
				var startCollectionUpdate = {};

				startCollectionUpdate.message = this.uploadStartMessage;
				startCollectionUpdate.progress = "0";
				
				this.updateStatusNow(dojo.toJson(startCollectionUpdate), false);
				var uiFormInput =  this.uploadInterface.getFormInput();
				uiFormInput.parms.fileToUpload = this._archiveLocation;
				
				dojo.publish("TransferCollection", [uiFormInput]);
				this.uploadInterface.disableSend("true");
				this.uploadInterface.hide();
				this.enableCancelButton();
			}));
			
			dojo.connect(this.uploadInterface.doNotUploadButtonNode, "onClick", dojo.hitch(this, function(){
				
				this.doNotUpload();
				this.enableRestartButton();
				this.enableRestartUploadButton();
				console.debug("RestartUploadButton called: 3");
			}));
			
			dojo.byId("collectionTransferDiv").appendChild(this.uploadInterface.domNode);

			dojo.byId("testthis").focus();
			console.debug("The focus has been placed on the collection status");
		}
		else {
			dojo.publish("ShowUploadInterface", []);
			dojo.byId("testthis").focus();
			console.debug("The focus has been placed on the collection status again");
		}
	},
	
	//Disables the ability to upload
	doNotUpload: function(){
		var statusJSON = {};
		statusJSON.type = "UPLOAD";
		statusJSON.collectionStatus = {};
		statusJSON.collectionStatus.noUpload = true;
		statusJSON.collectionStatus.completionStatus = "CANCELED";  
		dojo.publish("UploadCompleted", [dojo.toJson(statusJSON)]);	 
	},
	
	enableRestartButton: function() {
		dojo.publish("EnableRestart", []);	
	},
	
	enableRestartUploadButton: function() {
		dojo.publish("EnableUploadRestart", []);	
	},
	
	enableCancelButton: function() {
		dojo.publish("EnableCancel", []);	
	},
	
	//Set the Archive File location
	setArchiveLocation: function(archiveLocation) {
		this._archiveLocation = archiveLocation;
	}
	
});